/* pisa_sane_scan.h					-*- C++ -*-
   Copyright (C) 2001, 2004, 2008 SEIKO EPSON Corporation

   This file is part of the `iscan' program.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   As a special exception, the copyright holders give permission
   to link the code of this program with the esmod library and
   distribute linked combinations including the two.  You must obey
   the GNU General Public License in all respects for all of the
   code used other then esmod.
*/

#ifndef ___PISA_SANE_SCAN_H
#define ___PISA_SANE_SCAN_H

#ifdef __cplusplus
extern "C" {
#endif

#include <sane/sane.h>
#include <sane/saneopts.h>

#undef  SANE_NAME_SCAN_X_RESOLUTION
#define SANE_NAME_SCAN_X_RESOLUTION "x-resolution"

#ifdef __cplusplus
}
#endif

#include "pisa_enums.h"
#include "pisa_structs.h"

typedef struct
{
  char			option;
  char			film;
  char			pixeltype;
  char			bitdepth;
  char			scanspeed;
  char			dropout;
  char			monoopt;
  char			halftone;

  double		offset[2];
  double		area[2];
  long			resolution[2];
  long			zoom[2];

  gamma_struct		gamma;
  double		coef[9];

  long			threshold;
  long			speed;
  long			focus;
  long			usm;
  long			de_screening;

} scan_parameter;


class sane_scan
{
 public:
  void	close_device (void);

  // queries
  void	get_current_max_size (double *width, double *height) const;
  long	get_max_resolution (long cutoff = -1) const;
  void	get_color_profile (double *coef) const;
  bool	area_is_too_large (const scan_parameter& param) const;
  bool	has_adf (void) const;
  bool	has_tpu (void) const;
  bool	has_zoom (void) const;
  bool	has_focus (void) const;
  bool	has_start_button (void) const;
  bool	is_dumb (void) const;
  bool	is_button_pressed (void) const;

  // mutators
  void	set_option (pisa_option_type option);
  void	set_film_type (pisa_film_type film);

 protected:
  void	init (void);
  void	open_device (char *name = 0);

  void	set_scan_parameter (const scan_parameter& param);
  void	start_scan (int *width, int *height);
  void	acquire_image (unsigned char *img, int row_bytes, int height,
		       int cancel);
  void	finish_acquire (void);

  int	is_activate (const char *option_name) const;
  long	get_resolution (SANE_String_Const direction, int min_res) const;
  long	get_max_resolution (const char *option_name, long cutoff = -1) const;
  int	get_option_id (const char *option_name) const;
  void	get_value (const char *option_name, void *value) const;

  void	set_value (const char *option_name, void *value);

 private:
  void	update_settings (const scan_parameter& param);

  void	query_device (char *device_name) const;
  void	get_scanner_info (const char *name = 0);

  void	set_color_mode (char pixeltype, char bitdepth);
  void	set_color_profile (const double *coef);
  void	set_focus (long position);
  void	set_gamma_table (const unsigned char *gamma_table);
  void	set_scan_area (double offset_x, double offset_y,
		       double width, double height);
  void	set_scan_resolution (int resolution_x, int resolution_y);
  void	set_scan_zoom (int zoom_x, int zoom_y);
  void	set_speed (long speed);
  void	set_threshold (long threshold);

 private:
  SANE_Handle		m_hdevice;
  SANE_Parameters	m_sane_para;
  long			m_rows;

  char *		name;
  pisa_option_type	support_option;
  bool			support_zoom;
  bool			support_focus;
  bool			support_start_button;
  long			max_resolution;
};


#endif // ___PISA_SANE_SCAN_H
